<?php

namespace HCM\FPA\Entity;

use HCM\Util\Repositories;
use HCM\Util\TkEntity;
use HCM\Util\DateUtil;

class FpadirfmagnetEntity extends TkEntity
{
    public function __construct($entityManager, $hcmParameters){

        parent::__construct(null, $entityManager);
        $this->hcmParameters = $hcmParameters;
        $this->dirfMagnet = Repositories::FPA_DIRFMAGNET;
    }
    
    public function retornaRegistros($nrOrg, $mesCompetencia, $cdInscEstrut){
        return $this->entityManager->getRepository(Repositories::FPA_DIRFMAGNET)->retornaRegistrosPorInscricao($nrOrg, $mesCompetencia, $cdInscEstrut);
    }

    public function inicializaListaRegistros($nrOrg, $mesCompetencia, $cdInscEstrut){
        $resultados = $this->entityManager->getRepository(Repositories::FPA_DIRFMAGNET)->retornaRegistrosPorInscricao($nrOrg, $mesCompetencia, $cdInscEstrut);
        if (is_array($resultados) === true) {
            foreach ($resultados as $resultado) {
                $this->addItem($resultado, $resultado->dirfmagnet());
            }
            return true;
        } else {
            return false;
        }
    }

    public function insereRegistroLista(
        $nrOrg, $mesCompetencia, $tipoRegistro, $tipoInscricao, $nrInscricao, $linha, $cdoperador,
        $tipoInscricaoTom = "0", $nrInscricaoTom = "0", $nrVinculo = null, $dtAdmissao = null, $nrPisPasepCi = "0",
        $vrSalarioFamilia = null, $vrSalarioMaternidade = null, $categoriaTrabalhador = null, $nmrazaoestrut = null
    ){

        $item = new $this->dirfMagnet;

        $item->setIdativo('S');
        $item->setNrorg($nrOrg);
        $item->setMescompetencia($mesCompetencia);
        $item->setIdtiporegistro($tipoRegistro);
        $item->setIdtpinscestrutu($tipoInscricao);
        $item->setCdinscestrut($nrInscricao);
        $item->setDslinharegistro($linha);

        $item->setIdtpinscesttoma($tipoInscricaoTom);
        $item->setCdinscestrutoma($nrInscricaoTom);
        $item->setNrvinculom($nrVinculo);
        $item->setDtadmissaovinc($dtAdmissao);
        $item->setNrpispaseppes($nrPisPasepCi);
        $item->setVrsalfamilia($vrSalarioFamilia);
        $item->setVrsalmaternidad($vrSalarioMaternidade);
        $item->setIdcategtrab($categoriaTrabalhador);
        $item->setNmrazsocestrut($nmrazaoestrut);
        
        $item->setDtinclusao(DateUtil::getDataAtual());
        $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $item->setDtultatu(DateUtil::getDataAtual());
        $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $item->setCdoperultatu($this->hcmParameters->getCdoperador());
            
        $item->dirfmagnet(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DIRFMAGNET', 1, 12, true));

        $this->addItem($item);
    }

    public function persisteRegistro(
        $nrOrg, $cdOperador, $mesCompetencia, $tipoRegistro, $tipoInscricao, $nrInscricao, $tipoInscricaoTomador,
        $nrInscricaoTomador, $linha, $nrVinculoM, $dataAdmissao, $nrPisPasepCi, $vrSalarioFamilia,
        $vrSalarioMaternidade, $categoriaTrabalhador, $nmrazaoestrut = null
    ){
        $item = $this->entityManager->getRepository(Repositories::FPA_DIRFMAGNET)->retornaRegistrosTrabalhador(
            $mesCompetencia, $nrOrg, $tipoRegistro, $tipoInscricao, $nrInscricao, $tipoInscricaoTomador, $nrInscricaoTomador,
            $nrPisPasepCi, $dataAdmissao, $categoriaTrabalhador, $cdOperador
        );
        
        if (is_object($item) === true) {
            $item->setVrsalfamilia($vrSalarioFamilia);
            $item->setVrsalmaternidad($vrSalarioMaternidade);
            $item->setDslinharegistro($linha);
        } else {
            $item = new $this->dirfMagnet;

            $item->setIdativo('S');
            $item->setNrorg($nrOrg);
            $item->setMescompetencia($mesCompetencia);
            $item->setIdtiporegistro($tipoRegistro);
            $item->setIdtpinscestrutu($tipoInscricao);
            $item->setCdinscestrut($nrInscricao);
            $item->setIdtpinscesttoma($tipoInscricaoTomador);
            $item->setCdinscestrutoma($nrInscricaoTomador);
            $item->setNrvinculom($nrVinculoM);
            $item->setDtadmissaovinc($dataAdmissao);
            $item->setNrpispaseppes($nrPisPasepCi);
            $item->setVrsalfamilia($vrSalarioFamilia);
            $item->setVrsalmaternidad($vrSalarioMaternidade);
            $item->setIdcategtrab($categoriaTrabalhador);
            $item->setDslinharegistro($linha);
            $item->setNmrazsocestrut($nmrazaoestrut);
            $item->setDtinclusao(DateUtil::getDataAtual());
            $item->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $item->setDtultatu(DateUtil::getDataAtual());
            $item->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $item->setCdoperultatu($this->hcmParameters->getCdoperador());
            $item->dirfmagnet(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_DIRFMAGNET', 1, 12, true));
        }
        
        $this->entityManager->persist($item);
        $this->entityManager->flush($item);
        
    }

    public function procuraRegistroLista($nrInscricao, $nrInscricaoTom, $nrRegistro, $nrorg, $cdoperador) {
        $itens = $this->getItems();
        foreach ($itens as $key => $item) {
            if (($item->getCdinscestrut() === $nrInscricao)
                && ($item->getCdinscestrutoma() === $nrInscricaoTom)
                && ($item->getIdtiporegistro() === $nrRegistro)
                && ($item->getCdoperinclusao() == $cdoperador)
                && ($item->getNrorginclusao() == $nrorg)
            ) {
                return $key;
            }
        }    
        return false;
    }
    
    public function retroceder($nrOrg, $mesCompetencia, $cdoperador){

        $registros = $this->retornaRegistros($mesCompetencia, $nrOrg, $cdoperador);
        if (count($registros) > 0) {
            foreach ($registros as $registro) {
                $this->addItem($registro);
                $this->removeItem($registro);
            }
            $this->persist();
        }
    }
}