/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/******************************************************************************/
/* Nome: CALC_ABASTECIMENTO                                                   */
/*                                                                            */
/* Descrio:                                                                 */
/* Calcula datas de abastecimento para um determinado mes baseando-se na      */
/* Periodicidade de abastecimento e nos Feriados.                             */
/*                                                                            */
/* Criao:                                                                   */
/* 03/07/2002 - Janaina Grazielly C. Hanashiro                                */
/******************************************************************************/
/* PARAMETROS:                                                                */
/*                                                                            */
/* P_CDFILIAL     - Cdigo da Filial                                          */
/* P_CDCLIENTE    - Cdigo do Cliente                                         */
/* P_CDROTATRANS  - Cdigo da Rota                                            */
/* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        */
/* P_DTINICIAL    - Data do Inicio do Mes                                     */
/* P_NRMESES      - Numero de meses a serem calculados                        */
/******************************************************************************/
/* @C:\WIN_PRJ\DBA\SQL\ORACLE\8670_VMNSTP_CALC_ABASTECIMENTO.SQL              */
/******************************************************************************/
CREATE OR REPLACE
PROCEDURE  CALC_ABAST_ENTREGA
  (P_CDFILIAL     IN VARCHAR2,
   P_CDPERIENTR   IN VARCHAR2,
   P_CDROTATRANS  IN VARCHAR2,
   P_DTINICIAL    IN DATE,
   P_NRMESES      IN NUMBER) IS
   V_CONTAMES   NUMBER;
   V_IDTIPODIA  VARCHAR2(1);
   V_IDDIANUTIL VARCHAR2(1);
   V_DTINIAUX   DATE;
   V_DTCALEABAS DATE;
   V_CONTA      NUMBER;
BEGIN
  SELECT IDTIPODIA,IDDIANUTIL
  INTO   V_IDTIPODIA, V_IDDIANUTIL
  FROM   PERIENTR
  WHERE  CDPERIENTR = P_CDPERIENTR;

  V_DTINIAUX := P_DTINICIAL;
  FOR V_CONTAMES IN 1..P_NRMESES LOOP
    V_CONTA := 1;

    SELECT COUNT(*) INTO V_CONTA
    FROM CALENTREGA
    WHERE CDROTATRANS   = P_CDROTATRANS AND
          TO_CHAR(DTCALENTREGA,'MM/YYYY') = TO_CHAR(V_DTINIAUX,'MM/YYYY');

    IF V_CONTA > 0 THEN
       DELETE FROM CALENTREGA
       WHERE  CDROTATRANS   = P_CDROTATRANS AND
              TO_CHAR(DTCALENTREGA,'MMYYYY') = TO_CHAR(V_DTINIAUX,'MMYYYY');
    END IF;

    IF V_IDTIPODIA = 'M' THEN
       MES_ABAST_ENTREGA(P_CDFILIAL,P_CDROTATRANS,P_CDPERIENTR,V_DTINIAUX,V_IDDIANUTIL);
    ELSE
       IF V_IDTIPODIA = 'S' THEN
          SEM_ABAST_ENTREGA(P_CDFILIAL,P_CDROTATRANS,P_CDPERIENTR,V_DTINIAUX,V_IDDIANUTIL);
       END IF;   
    END IF;

    V_DTINIAUX := LAST_DAY(V_DTINIAUX) + 1;

  END LOOP;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CALC_ABAST_ENTREGA','4.11.151'); END;
/

